#ifndef ideal_t
#define ideal_t
#include "poly.h"
#include "contain.t"

/////////////////
// DEFINITIONS //
/////////////////

bool Auto_Elim = false;

/*********/
/* ideal */
/*********/

template<class numeric> class ideal {

  friend ostream& operator<<(ostream&, const ideal<numeric>&);

  public:

    typedef poly<numeric> numpoly;

    unsigned long gencount;
    list<numpoly> gens    ;

    // constructors:

    // standard constructor:
    ideal():
      gencount(0            ) ,
      gens    (0, numpoly(0)) ,
      autoelim(Auto_Elim    ) {
      if (autoelim)
        Normalize();
    }

    // general constructors:
    ideal(const numeric &a):
      gencount(1            ) ,
      gens    (1, numpoly(a)) ,
      autoelim(Auto_Elim    ) {
      if (autoelim)
        Normalize();
    }

    ideal(const numpoly &p):
      gencount(1        ) ,
      gens    (1, p     ) ,
      autoelim(Auto_Elim) {
      if (autoelim)
        Normalize();
    }

    ideal(const list<numpoly> &l):
      gencount(l.length()) ,
      gens    (l         ) ,
      autoelim(Auto_Elim ) {
      if (autoelim)
        ElimMultiples();
    }

    // internal operators:
    ideal operator+=(const ideal&);
    ideal operator*=(const ideal&);
    ideal operator =(const ideal&);

    // methods:
    bool ishomo() const;
    void SetAutoElim(const bool&);

  private:
    bool autoelim;

    // methods:
    void ElimMultiples();
    void Normalize    ();
};

// external operators:
template<class numeric> ideal<numeric> operator+ (const ideal<numeric>&, const ideal<numeric>&);
template<class numeric> ideal<numeric> operator* (const ideal<numeric>&, const ideal<numeric>&);
template<class numeric> ostream&       operator<<(ostream             &, const ideal<numeric>&);



/********************/
/* homogenous ideal */
/********************/

template<class numeric> class homoideal {

  friend ostream& operator<<(ostream&, const homoideal<numeric>&);

  public:

    typedef homopoly<numeric> numhomopoly;

    unsigned long gencount;
    list<numhomopoly> gens;

    // constructors:

    // standard constructor:
    homoideal():
      gencount(0                ) ,
      gens    (0, numhomopoly(0)) ,
      autoelim(Auto_Elim        ) {
      if (autoelim)
        Normalize();
    }

    // general constructors:
    homoideal(const numeric &a):
      gencount(1                ) ,
      gens    (1, numhomopoly(a)) ,
      autoelim(Auto_Elim        ) {
      if (autoelim)
        Normalize();
    }

    homoideal(const numhomopoly &p):
      gencount(1        ) ,
      gens    (1, p     ) ,
      autoelim(Auto_Elim) {
      if (autoelim)
        Normalize();
    }

    homoideal(const list<numhomopoly> &l):
      gencount(l.length()) ,
      gens    (l         ) ,
      autoelim(Auto_Elim ) {
      if (autoelim)
        ElimMultiples();
    }

    homoideal(const ideal<numeric> &I):
      gencount(I.gencount                ) ,
      gens    (I.gencount, numhomopoly(0)) ,
      autoelim(Auto_Elim                 ) {
      for (unsigned int i = 1; i <= gencount; i++)
        gens[i] = numhomopoly(I.gens[i]);
      if (autoelim)
        ElimMultiples();
    }

    // internal operators:
    homoideal operator+=(const homoideal&);
    homoideal operator*=(const homoideal&);
    homoideal operator =(const homoideal&);

    // type conversion:

    operator ideal<numeric>() {
      list<poly<numeric> > L(gens.length(), poly<numeric>());
      for (unsigned i = 1; i <= gens.length(); i++)
        L[i] = gens[i];
      return ideal<numeric>(L);
    }

    // methods:
    void SetAutoElim(const bool&);

  private:
    bool autoelim;

    // methods:
    void ElimMultiples();
    void Normalize    ();
};

// external operators:
template<class numeric> homoideal<numeric> operator+ (const homoideal<numeric>&, const homoideal<numeric>&);
template<class numeric> homoideal<numeric> operator* (const homoideal<numeric>&, const homoideal<numeric>&);
template<class numeric> ostream&           operator<<(ostream                 &, const homoideal<numeric>&);



///////////////////
//IMPLEMENTATIONS//
///////////////////


/*********/
/* ideal */
/*********/

// operators

// addition operators:

template<class numeric> ideal<numeric> operator+(ideal<numeric> &I, ideal<numeric> &J) {
  ideal<numeric> temp = I;
  temp += J;
  return temp;
}

template<class numeric> ideal<numeric> ideal<numeric>::operator+=(const ideal<numeric> &I) {
  gencount += I.gencount;
  gens     += I.gens    ;
  ElimMultiples();
  return *this;
}

// multiplication operators:

template<class numeric> ideal<numeric> operator*(ideal<numeric> &I, ideal<numeric> &J) {
  ideal<numeric> temp = I;
  temp *= J;
  return temp;
}

template<class numeric> ideal<numeric> ideal<numeric>::operator*=(const ideal<numeric> &I) {
  unsigned long oldgencount = gencount, i, j;
  list<numpoly> oldgens     = gens          ;
  gencount *= I.gencount;
  gens = list<numpoly>(gencount, numpoly());
  for (i = 1; i <= oldgencount; i++)
    for (j = 1; j <= I.gencount; j++)
      gens[(i - 1) * I.gencount + j] = oldgens[i] * I.gens[j];
  ElimMultiples();
  return *this;
}

// assignment operator:

template<class numeric> ideal<numeric> ideal<numeric>::operator=(const ideal<numeric> &I) {
  gencount = I.gencount;
  gens     = I.gens    ;
  return *this;
}

// output operator:

template<class numeric> ostream& operator<<(ostream &out, const ideal<numeric> &I) {
  out << "Ideal(";
  if (I.gencount == 0)
    out << 0;
  else
    for (unsigned long i = 1; i <= I.gencount; i++) {
      out << I.gens[i];
      if (i < I.gencount)
        out << ", ";
    }
  out << ")";
}

// methods:

template<class numeric> void ideal<numeric>::ElimMultiples() {
  unsigned long i, j;
  numpoly zeropoly;
  for (i = 1; i <= gencount; i++)
    for (j = 1; j <= gencount; j++)
      if (i != j && gens[j] % gens[i] == zeropoly) {
        gens.remove(j);
        gencount--;
        j       --;
        if (i > j)
          i--;
      }
  Normalize();
}

template<class numeric> bool ideal<numeric>::ishomo() const {
  for (unsigned long i = 1; i <= gencount; i++)
    if (!gens[i].ishomo())
      return false;
  return true;
}

template<class numeric> void ideal<numeric>::Normalize() {
  numeric dummy;
  if (dummy.classtype() == "field")
    for (unsigned long i = 1; i <= gencount; i++)
      gens[i] = normal(gens[i]);
}

template<class numeric> void ideal<numeric>::SetAutoElim(const bool &set) {
  if (autoelim = set)
    ElimMultiples();
}


/********************/
/* homogenous ideal */
/********************/

// operators

// addition operators:

template<class numeric> homoideal<numeric> operator+(homoideal<numeric> &I, homoideal<numeric> &J) {
  homoideal<numeric> temp = I;
  temp += J;
  return temp;
}

template<class numeric> homoideal<numeric> homoideal<numeric>::operator+=(const homoideal<numeric> &I) {
  gencount += I.gencount;
  gens     += I.gens    ;
  ElimMultiples();
  return *this;
}

// multiplication operators:

template<class numeric> homoideal<numeric> operator*(homoideal<numeric> &I, homoideal<numeric> &J) {
  homoideal<numeric> temp = I;
  temp *= J;
  return temp;
}

template<class numeric> homoideal<numeric> homoideal<numeric>::operator*=(const homoideal<numeric> &I) {
  unsigned long oldgencount = gencount, i, j;
  list<numhomopoly> oldgens = gens          ;
  gencount *= I.gencount;
  gens = list<numhomopoly>(gencount, numhomopoly());
  for (i = 1; i <= oldgencount; i++)
    for (j = 1; j <= I.gencount; j++)
      gens[(i - 1) * I.gencount + j] = oldgens[i] * I.gens[j];
  ElimMultiples();
  return *this;
}

// assignment operator:

template<class numeric> homoideal<numeric> homoideal<numeric>::operator=(const homoideal<numeric> &I) {
  gencount = I.gencount;
  gens     = I.gens    ;
  return *this;
}

// output operator:

template<class numeric> ostream& operator<<(ostream &out, const homoideal<numeric> &I) {
  out << "Ideal(";
  if (I.gencount == 0)
    out << 0;
  else
    for (unsigned long i = 1; i <= I.gencount; i++) {
      out << I.gens[i];
      if (i < I.gencount)
        out << ", ";
    }
  out << ")";
}

// methods:

template<class numeric> void homoideal<numeric>::ElimMultiples() {
  unsigned long i, j;
  numhomopoly zeropoly;
  for (i = 1; i <= gencount; i++)
    for (j = 1; j <= gencount; j++)
      if (i != j && gens[j] % gens[i] == zeropoly) {
        gens.remove(j);
        gencount--;
        j       --;
        if (i > j)
          i--;
      }
  Normalize();
}

template<class numeric> void homoideal<numeric>::Normalize() {
  numeric dummy;
  if (dummy.classtype() == "field")
    for (unsigned long i = 1; i <= gencount; i++)
      gens[i] = normal(gens[i]);
}

template<class numeric> void homoideal<numeric>::SetAutoElim(const bool &set) {
  if (autoelim = set)
    ElimMultiples();
}

#endif

