#ifndef sparse_t
#define sparse_t

/////////////////
// DEFINITIONS //
/////////////////

/*********/
/* entry */
/*********/

template<class numeric> class sparseentry {
  public:
    unsigned long position;
    numeric       value   ;
    sparseentry*  next    ;

    // constructors:

    // standard constructor:
    sparseentry():
      position(0   ) ,
      value   (0   ) ,
      next    (NULL) {
    }

    // general constructors:
    sparseentry(unsigned long pos, numeric a):
      position(pos ) ,
      value   (a   ) ,
      next    (NULL) {
    }
    sparseentry(sparseentry* p):
      position(0) ,
      value   (0) ,
      next    (p) {
    }
    sparseentry(unsigned long &pos, numeric &a, sparseentry* p):
      position(pos) ,
      value   (a  ) ,
      next    (p  ) {
    }
};

/**********/
/* vector */
/**********/

template<class numeric> class sparsevector {

  public:
    sparseentry<numeric>* first  ;
    sparseentry<numeric>* last   ;

    // constructors:

    // general constructors:
    sparsevector():
      first     (NULL) ,
      last      (NULL) ,
      size      (0   ) ,
      entrycount(0   ) {
    }
    sparsevector(const unsigned long &n):
      first     (NULL) ,
      last      (NULL) ,
      size      (n   ) ,
      entrycount(0   ) {
    };
    sparsevector(const unsigned long &n, const numeric &ini):
      first(NULL) ,
      last (NULL) ,
      size(n)     {
      if (ini == 0)
        entrycount = 0;
      else {
        entrycount = n;
        init(ini);
      }
    };

    // copy constructor:
    sparsevector(const sparsevector &v):
      first(NULL) {
      size       = v.size      ;
      entrycount = v.entrycount;
      if (v.first) {
        first = new sparseentry<numeric>;
        sparseentry<numeric> *entry = first, *ventry = v.first;
        entry->position = ventry->position;
        entry->value    = ventry->value   ;
        while (ventry->next) {
          entry->next     = new sparseentry<numeric>;
          entry           = entry ->next            ;
          ventry          = ventry->next            ;
          entry->position = ventry->position        ;
          entry->value    = ventry->value           ;
        }
      }
    }

    // destructor:
    ~sparsevector() {
      if (first) {
        sparseentry<numeric> *entry = first, *nextentry = entry->next;
        while (nextentry) {
          delete entry;
          entry     = nextentry  ;
          nextentry = entry->next;
        }
        delete entry;
      }
    }

    // internal operators:
     sparsevector& operator+=(const sparsevector &)      ;
     sparsevector& operator-=(const sparsevector &)      ;
     sparsevector& operator*=(const numeric      &)      ;
     sparsevector& operator/=(const numeric      &)      ;
          numeric  operator[](const unsigned long&)      ;
    const numeric  operator[](const unsigned long&) const;
     sparsevector& operator =(const sparsevector &)      ;

    //methods:
    void          add    (unsigned long, numeric   );
    void          append (unsigned long, numeric   );
    void          clear  (unsigned long newsize = 0);
    unsigned long entries(                         ) const {return entrycount;}
    void          insert (unsigned long, numeric   );
    bool          isnull (                         ) const             ;
    unsigned long length (                         ) const {return size;}
    void          remove (unsigned long            );

  private:
    unsigned long size, entrycount;

    // methods:
    void init(const numeric &a) {
      if (size > 0) {
        first = new sparseentry<numeric>(1, a);
        last = first;
        for (unsigned long i = 2; i <= size; i++) {
          last->next = new sparseentry<numeric>(i, a);
          last       = last->next                    ;
        }
      }
    }
};


// external operators:
template<class numeric> sparsevector<numeric> operator+ (const sparsevector<numeric>&                              );
template<class numeric> sparsevector<numeric> operator- (const sparsevector<numeric>&                              );
template<class numeric> sparsevector<numeric> operator+ (const sparsevector<numeric>&, const sparsevector<numeric>&);
template<class numeric> sparsevector<numeric> operator- (const sparsevector<numeric>&, const sparsevector<numeric>&);
template<class numeric> numeric               operator* (const sparsevector<numeric>&, const sparsevector<numeric>&);
template<class numeric> sparsevector<numeric> operator* (const sparsevector<numeric>&, const numeric              &);
template<class numeric> sparsevector<numeric> operator* (const numeric              &, const sparsevector<numeric>&);
template<class numeric> sparsevector<numeric> operator/ (const sparsevector<numeric>&, const numeric              &);
template<class numeric> bool                  operator==(const sparsevector<numeric>&, const sparsevector<numeric>&);
template<class numeric> bool                  operator!=(const sparsevector<numeric>&, const sparsevector<numeric>&);
template<class numeric> ostream&              operator<<(      ostream              &, const sparsevector<numeric>&);

/**********/
/* matrix */
/**********/

template<class numeric> class sparsematrix {

  public:
    // constructors:

    // general constructors:
    sparsematrix():
      height(0) ,
      width (0) ,
      rows  (list<sparsevector<numeric>*>()) {
    }
    sparsematrix(const unsigned long &m, const unsigned long &n):
      height(m) ,
      width (n) {
      sparsevector<numeric> *null = NULL;
      rows = list<sparsevector<numeric>*>(m, null);
      for (unsigned long i = 1; i <= m; i++)
        rows[i] = new sparsevector<numeric>(n);
    };

    // copy constructor:
    sparsematrix(const sparsematrix &A):
      height(A.height) ,
      width (A.width ) {
      sparsevector<numeric> *null = NULL;
      rows = list<sparsevector<numeric>*>(A.height, null);
      for (unsigned long i = 1; i <= height; i++) {
        rows[i]  = new sparsevector<numeric>();
        *rows[i] = *A.rows[i];
      }
    }

    // destructor:
    ~sparsematrix() {
      for (unsigned long i = 1; i <= height; i++)
        delete rows[i];
    }

    // internal operators:
          sparsevector<numeric> operator[](const unsigned long&)      ;
    const sparsevector<numeric> operator[](const unsigned long&) const;
          sparsematrix&         operator= (const sparsematrix &)      ;

    //methods:
    void          appendrow  (const sparsevector<numeric>&);
    void          clear      ();
    void          Gauss      (const unsigned long&);
    unsigned long getheight  () const {return height;}
    double        getsparsity();
    unsigned long getwidth   () const {return width ;}
    void          removerow  (const unsigned long&);
    void          removezeros();
    void          RowEchelon ();
    void          swaprows   (const unsigned long&, const unsigned long&);

  private:
    unsigned long height, width;
    list<sparsevector<numeric>*> rows;
};




///////////////////
//IMPLEMENTATIONS//
///////////////////

/**********/
/* vector */
/**********/

// operators:

// addition and substraction operators:

template<class numeric> sparsevector<numeric> operator+(const sparsevector<numeric> &v) {
  return v;
}

template<class numeric> sparsevector<numeric> operator-(const sparsevector<numeric> &v) {
  sparsevector<numeric> temp   = v         ;
  sparseentry <numeric> *entry = temp.first;
  while (entry) {
    entry->value = -(entry->value);
    entry        = entry->next    ;
  }
  return temp;
}

template<class numeric> sparsevector<numeric> operator+(const sparsevector<numeric> &v, const sparsevector<numeric> &w) {
  sparsevector<numeric> temp = v;
  temp += w;
  return temp;
}

template<class numeric> sparsevector<numeric> operator-(const sparsevector<numeric> &v, const sparsevector<numeric> &w) {
  sparsevector<numeric> temp = v;
  temp -= w;
  return temp;
}

template<class numeric> sparsevector<numeric>& sparsevector<numeric>::operator+=(const sparsevector<numeric> &v) {
  sparseentry<numeric> *entry = first, *ventry = v.first;
  if (size != v.size)
    Error(VecSizeDiff, "sparsevector+=sparsevector");
  last = NULL;
  while (entry != NULL && ventry != NULL) {
    if (entry->position > ventry->position) {
      if (last) {
        last->next = new sparseentry<numeric>(ventry->position, ventry->value, entry);
        last = last->next;
      }
      else {
        first = new sparseentry<numeric>(ventry->position, ventry->value, entry);
        last  = first;
      }
      entrycount++;
      ventry = ventry->next;
    }
    else if (entry->position == ventry->position) {
      entry->value += ventry->value;
      if (entry->value == 0) {
        if (last) {
          entry = entry->next;
          delete last->next;
          last->next = entry;
        }
        else {
          entry = entry->next;
          delete first;
          first = entry;
        }
        entrycount--;
      }
      else {
        last   = entry       ;
        entry  = entry ->next;
      }
      ventry = ventry->next;
    }
    else {
      last  = entry      ;
      entry = entry->next;
    }
  }
  while (entry) {
    last  = entry      ;
    entry = entry->next;
  }
  while (ventry) {
    if (last) {
      last->next = new sparseentry<numeric>(ventry->position, ventry->value);
      last       = last->next;
    }
    else {
      first = new sparseentry<numeric>(ventry->position, ventry->value);
      last  = first;
    }
    entrycount++;
    ventry = ventry->next;
  }
  return *this;
}

template<class numeric> sparsevector<numeric>& sparsevector<numeric>::operator-=(const sparsevector<numeric> &v) {
  *this += -v;
  return *this;
}

// multiplication and division operators:

template<class numeric> numeric operator*(const sparsevector<numeric> &v, const sparsevector<numeric> &w) {
  if (v.length() != w.length())
    Error(VecSizeDiff, "sparsevector*sparsevector");
  numeric sum = 0;
  sparseentry<numeric> *ventry = v.first, *wentry = w.first;
  while (v.first != NULL && w.first != NULL)
    if (v->position < w->position)
      ventry = ventry->next;
    else if (v->position > w->position)
      wentry = wentry->next;
    else  {
      sum += ventry->value * wentry->value;
      ventry = ventry->next;
      wentry = wentry->next;
    }
  return sum;
}

template<class numeric> sparsevector<numeric> operator*(const sparsevector<numeric> &v, const numeric &a) {
  sparsevector<numeric> temp = v;
  temp *= a;
  return temp;
}

template<class numeric> sparsevector<numeric> operator*(const numeric &a, const sparsevector<numeric> &v) {
  sparsevector<numeric> temp = v;
  temp *= a;
  return temp;
}

template<class numeric> sparsevector<numeric> operator/(const sparsevector<numeric> &v, const numeric &a) {
  sparsevector<numeric> temp = v;
  temp /= a;
  return temp;
}

template<class numeric> sparsevector<numeric>& sparsevector<numeric>::operator*=(const numeric &a) {
  if (a == 0)
    clear();
  else {
    sparseentry<numeric> *entry = first;
    while(entry) {
      entry->value *= a;
      entry = entry->next;
    }
  }
  return *this;
}

template<class numeric> sparsevector<numeric>& sparsevector<numeric>::operator/=(const numeric &a) {
  sparseentry<numeric> *entry = first;
  if (a == 0)
    Error(DivByZero, "sparsevector/=numeric");
  else
    while(entry) {
      entry->value /= a;
      entry = entry->next;
    }
  return *this;
}

// comparison operators:

template<class numeric> bool operator==(const sparsevector<numeric> &v, const sparsevector<numeric> &w) {
  sparseentry<numeric> *ventry = v.first, *wentry = w.first;
  if (v.length() != w.length())
    Error(VecSizeDiff, "sparsevector==sparsevector");
  if (v.entries() != w.entries())
    return false;
  while (ventry != NULL && wentry != NULL) {
    if (ventry->position != wentry->position || ventry->value != wentry->value)
      return false;
    ventry = ventry->next;
    wentry = wentry->next;
  }
  if (ventry != NULL || wentry != NULL)
    return false;
  return true;
}

template<class numeric> bool operator!=(const sparsevector<numeric> &v, const sparsevector<numeric> &w) {
  return !(v == w);
}

// index operators:

template<class numeric> numeric sparsevector<numeric>::operator[](const unsigned long &i) {
  sparseentry<numeric> *entry = first;
  if (i == 0 || i > size)
    Error(IndexRange, "sparsevector[]");
  while(entry != NULL && entry->position < i)
    entry = entry->next;
  if (entry->position == i)
    return entry->value;
  return 0;
}

template<class numeric> const numeric sparsevector<numeric>::operator[](const unsigned long &i) const {
  sparseentry<numeric> *entry = first;
  if (i == 0 || i > size)
    Error(IndexRange, "vector[]");
  while(entry != NULL && entry->position < i)
    entry = entry->next;
  if (entry->position == i)
    return entry->value;
  return 0;
}

// assignment operator:

template<class numeric> sparsevector<numeric>& sparsevector<numeric>::operator=(const sparsevector<numeric> &v) {
  sparseentry<numeric> *entry = first, *ventry = v.first;
  if (this != &v) {
    size       = v.size      ;
    entrycount = v.entrycount;
    last = first ;
    while (entry != NULL && ventry != NULL) {
      last            = entry           ;
      entry->position = ventry->position;
      entry->value    = ventry->value   ;
      entry           = entry ->next    ;
      ventry          = ventry->next    ;
    }
    if (entry) {
      if (entry == first) {
        first = NULL;
        last  = NULL;
      }
      else
        last->next = NULL;
      while (entry) {
        ventry = entry->next;
        delete entry;
        entry = ventry;
      }

    }
    while (ventry) {
      if (last) {
        last->next = new sparseentry<numeric>(ventry->position, ventry->value);
        last       = last->next;
      }
      else {
        first = new sparseentry<numeric>(ventry->position, ventry->value);
        last  = first;
      }
      ventry = ventry->next;
    }
  }
  return *this;
}

// output operator:

template<class numeric> ostream& operator<<(ostream &out, const sparsevector<numeric> &v) {
  unsigned long len = v.length(), pos = 1;
  sparseentry<numeric> *entry = v.first;
  if (len == 0)
    out << "()";
  else {
    out << '(';
    if (entry->position == 1) {
      out << entry->value;
//      out << "[" << entry << "," << entry -> position << "," << entry->value << "," << entry->next <<"]";
      entry = entry->next;
    }
    else
      out << '0';
    while (entry) {
      for (pos++; pos < entry->position; pos++)
        out << ", 0";
      out << ", " << entry->value;
//      out << ", [" << entry << "," << entry -> position << "," << entry->value << "," << entry->next << "]";
      entry = entry->next;
    }
    for (pos++; pos <= len; pos++)
      out << ", 0";
    out << ')';
  }
  return out;
}


// methods:

template<class numeric> void sparsevector<numeric>::add(unsigned long pos, numeric a) {
  if (a == 0)
    return;
  if (!first) {
    first = new sparseentry<numeric>(pos, a);
    last  = first;
    entrycount++;
    return;
  }
  if (first->position > pos) {
    first = new sparseentry<numeric>(pos, a, first);
    entrycount++;
    return;
  }
  sparseentry<numeric> *entry = first, *lastentry;
  while (entry != NULL && entry->position < pos) {
    lastentry = entry      ;
    entry     = entry->next;
  }
  if (entry != NULL && entry->position == pos) {
    entry->value += a;
    if (entry->value == 0) {
      remove(pos);
      entrycount--;
    }
  }
  else {
    lastentry->next = new sparseentry<numeric>(pos, a, entry);
    if (!entry->next)
      last = entry;
    entrycount++;
  }
}

template<class numeric> void sparsevector<numeric>::append(unsigned long pos, numeric a) {
  if (a != 0) {
    last->next = new sparseentry<numeric>(pos, a);
    last       = last->next;
    if (!first)
      first = last;
    entrycount++;
  }
}

template<class numeric> void sparsevector<numeric>::clear(unsigned long newsize) {
  if (first) {
    sparseentry<numeric> *entry = first, *nextentry = entry->next;
    while (nextentry) {
      delete entry;
      entry     = nextentry  ;
      nextentry = entry->next;
    }
    delete entry;
  }
  first      = NULL;
  last       = NULL;
  entrycount = 0   ;
  if (newsize != 0)
    size = newsize;

}

template<class numeric> void sparsevector<numeric>::insert(unsigned long pos, numeric a) {
  if (first)
    if (pos <= first->position)
      if (pos == first->position)
        if(a == 0) {
          remove(pos);
          entrycount--;
        }
        else
          first->value = a;
      else {
        first = new sparseentry<numeric>(pos, a, first);
        entrycount++;
      }
    else {
      sparseentry<numeric> *entry = first;
      while (entry->next != NULL && entry->next->position < pos)
        entry = entry->next;
      if (entry->next != NULL && entry->next->position == pos)
        if(a == 0) {
          remove(pos);
          entrycount--;
        }
        else
          entry->next->value = a;
      else
        if (a != 0) {
          entry->next = new sparseentry<numeric>(pos, a, entry->next);
          if (!entry->next->next)
            last = entry->next;
          entrycount++;
        }
    }
  else {
    first = new sparseentry<numeric>(pos, a, NULL);
    entrycount = 1;
  }
}

template<class numeric> bool sparsevector<numeric>::isnull() const {
  return first == NULL;
}

template<class numeric> void sparsevector<numeric>::remove(unsigned long pos) {
  sparseentry<numeric> *entry = first, *lastentry, *nextentry;
  while (entry && entry->position < pos) {
    lastentry = entry      ;
    entry     = entry->next;
  }
  if (entry != NULL && entry->position == pos) {
    nextentry = entry->next;
    if (entry == first)
      first = nextentry;
    else
      lastentry->next = nextentry;
    delete entry;
    if (!nextentry)
      last = lastentry;
    entrycount--;
  }
}


/**********/
/* matrix */
/**********/

// operators:

// index operators:

template<class numeric> sparsevector<numeric> sparsematrix<numeric>::operator[](const unsigned long &i) {
  return *rows[i];
}

template<class numeric> const sparsevector<numeric> sparsematrix<numeric>::operator[](const unsigned long &i) const {
  return *rows[i];
}

// assignment operator:

template<class numeric> sparsematrix<numeric>& sparsematrix<numeric>::operator=(const sparsematrix<numeric> &A) {
  if (this != &A) {
    height = A.height;
    width  = A.width;
    rows   = list<sparsevector<numeric>*>(height);
    for (unsigned long i = 1; i <= height; i++) {
      rows[i]  = new sparsevector<numeric>;
      *rows[i] = *A.rows[i];
    }
  }
  return *this;
}

// output operator:

template<class numeric> ostream& operator<<(ostream &out, const sparsematrix<numeric> &M) {
  unsigned long l = M.getheight();
  for (unsigned long i = 1; i <= l; i++)
    out << endl << M[i];
  return out;
}


// methods:

template<class numeric> void sparsematrix<numeric>::appendrow(const sparsevector<numeric> &v) {
  sparsevector<numeric> *null = NULL;
  if (v.length() != width)
    if (height == 0)
      width = v.length();
    else
      Error(MatVecSizeDiff, "sparsematrix.appendrow()");
  height++;
  rows.append(null);
  rows[height] = new sparsevector<numeric>(v);
}

template<class numeric> void sparsematrix<numeric>::clear() {
  for (unsigned long i = 1; i <= height; i++)
    delete rows[i];
  rows.clear();
  height = 0;
}

template<class numeric> void sparsematrix<numeric>::Gauss(const unsigned long& startrow) {
  bool done;
  unsigned long i, j, row;
  sparsevector<numeric> temp;
  for (row = 1; row < height; row++)
    if (rows[row]->first)
      for (i = max(startrow, row + 1); i <= height; i++)
        if (rows[i]->first != NULL && rows[i]->first->position == rows[row]->first->position) {
          *rows[i] -= rows[i]->first->value / rows[row]->first->value * *rows[row];
          if (rows[i]->first != NULL && row >= startrow) {
            done = false;
            while (!done) {
              while (!done)
                for (j = min(rows[i]->first->position, startrow - 1), done = true; j > 0; j--)
                  if (rows[j]->first != NULL && rows[j]->first->position <= rows[i]->first->position) {
                    if (rows[j]->first->position == rows[i]->first->position) {
                      *rows[i] -= rows[i]->first->value / rows[j]->first->value * *rows[j];
                      done = false;
                    }
                    break;
                  }
              for (j = startrow; j < i; j++)
                if (rows[j]->first != NULL && rows[j]->first->position == rows[i]->first->position) {
                  temp = (rows[i]->first->value) / (rows[j]->first->value) * (*rows[j]);
                  *rows[i] -= temp;
                  done = false;
                  break;
                }
            }
          }
        }
  removezeros();
}

template<class numeric> double sparsematrix<numeric>::getsparsity() {
  unsigned long entrycount = 0;
  for (unsigned long i = 1; i <= height; i++)
    entrycount += rows[i]->entries();
  return 100. * entrycount / height / width;
}

template<class numeric> void sparsematrix<numeric>::removerow(const unsigned long &i) {
  if (i > 0 && i <= height) {
    delete rows[i];
    rows.remove(i);
    height--;
  }
}

template<class numeric> void sparsematrix<numeric>::removezeros() {
  for (unsigned long i = height; i > 0; i--)
    if (rows[i]->first == NULL)
      removerow(i);
}

template<class numeric> void sparsematrix<numeric>::RowEchelon() {
  unsigned long i, j, col, row;
  list<sparseentry<numeric>*> entries(height);
  sparseentry<numeric> *swap;
  for (i = 1; i <= height; i++)
    entries[i] = rows[i]->first;
  for (row = 1; row <= height; row++) {
    if (entries[row] == NULL) {
      removerow(row);
      entries.remove(row);
      row--;
    }
    else {
      col = entries[row]->position;
      j   = row                   ;
      for (i = row + 1; i <= height; i++) {
        if (entries[i]->position < col) {
          col = entries[i]->position;
          j   = i                   ;
        }
        else if (entries[i]->position == col) {
          if (rows[i]->entries() < rows[j]->entries())
            swaprows(i, j);
          removerow(i);
          entries.remove(i);
          i--;
        }
      }
      if (j != row) {
        swaprows(row, j);
        swap         = entries[row];
        entries[row] = entries[j  ];
        entries[j  ] = swap        ;
      }
    }
  }
}

template<class numeric> void sparsematrix<numeric>::swaprows(const unsigned long &i, const unsigned long &j) {
  if (i > 0 && j > 0 && i <= height && j <= height && i != j) {
    sparsevector<numeric> *swap = rows[i];
    rows[i] = rows[j];
    rows[j] = swap   ;
  }
}


#endif

